<?php
/**
 * Plugin Name: SmartHint - Smart Recommendations
 * Plugin URI: https://opencart.com/
 * Description: E-commerce Recommendation System increases the engagement and conversion rate by at least 35%. Like many online retailers, power up your sales serving to every customer as if they are unique. “For years, more than 35% of sales have been motivated by the recommendation system.” Jeff Bezos, CEO of Amazon.com.

 * Version: 1.0.0
 * Author: OpenCart
 * Author URI: http://www.smarthint.co/
 * Developer: Carlos Renan Damarate
 * Developer URI: http://www.smarthint.co/
 *
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

class ControllerExtensionModuleSmartHint extends Controller {
    
    public function index() {
        
        //Campos retornados para View
        $data = array();   

        //Carrega traduções em 'language/en-gb/...'
        $data = array_merge($data, $this->load->language('extension/module/smarthint'));
        
        //Carrega o módulo de produto
        $this->load->model('catalog/product');
        $this->load->model('setting/setting');

        //procura por user_token
        $token = "";
        if ( array_key_exists ( 'token' , $this->session->data ) ){
            $token = 'token=' . $this->session->data['token'];
    }
        if (empty($token)){
        $token = 'user_token=' . $this->session->data['user_token'];
        } 

        //Lê todos os produtos
        $this->document->addScript('view/javascript/smarthint/smarthint.js');
        
        //Verifica se foi enviado via POST (Salvar)
        if (($this->request->server['REQUEST_METHOD'] == 'POST')) {
            
            $enable = 0;
            if (isset($this->request->post['smarthint_module_status'])){
                $enable = $this->request->post['smarthint_module_status'];
            } else if (isset($this->request->post['module_smarthint_status'])){
                $enable = $this->request->post['module_smarthint_status'];
            }
            
            if ($enable){
                $this->model_setting_setting->editSetting('smarthint_module_status', $this->request->post);
                $this->model_setting_setting->editSetting('module_smarthint_status', $this->request->post);
                $this->session->data['success'] = $this->language->get('text_enable');
                $response = $this->create_smarthint_account();
                if ($response){
                    //self::registerEvents();
                    $this->response->redirect($this->url->link('extension/module/smarthint', $token . '&type=module', true));
                }
                else {
                    //$this->load->model('extension/event');
                    //$this->model_extension_event->deleteEvent("smarthint_module");
                    $data['error_warning'] = $this->language->get('error_account');
                }
            }
            else{
                //$this->load->model('extension/event');
                //$this->model_extension_event->deleteEvent("smarthint_module");
                $data['error_warning'] = $this->language->get('text_disable');
            }
        }

        
        //Configura Links de acesso
        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', $token, true)
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_extension'),
            'href' => $this->url->link('extension/extension', $token . '&type=module', true)
        );
  
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/module/smarthint_module', $token, true)
        );

        //Recupera informação de Enable/Disable
        //OC 2.3
        if (isset($this->request->post['smarthint_module_status'])) {
            $data['smarthint_module_status'] = $this->request->post['smarthint_module_status'];
		} else {
			$data['smarthint_module_status'] = $this->config->get('smarthint_module_status');
        }
        
        //OC 3.0.2.0
        if (isset($this->request->post['module_smarthint_status'])) {
            $data['module_smarthint_status'] = $this->request->post['module_smarthint_status'];
		} else {
			$data['module_smarthint_status'] = $this->config->get('module_smarthint_status');
        }

        //Ações dos botões
        $data['action'] = $this->url->link('extension/module/smarthint', $token, true);
        $data['cancel'] = $this->url->link('extension/extension', $token . '&type=module', true);
        //Carrega padrão da página
        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/module/smarthint_module', $data));
    }
    
    private function create_smarthint_account(){
        //Obtem URL da loja
        $baseSsl = $this->config->get('config_ssl');
        $base = $this->config->get('config_url');
        $url = (isset($baseSsl)) ? $baseSsl : $base;
        //Obtem dados da API
        $api = "empty";
        
        //Data config para criar conta
        $send = array(
            'Name' => $this->config->get('config_name'),
            'Mail' => $this->config->get('config_email'),
            'Domain' => (isset($url)) ? $url : "domain",
            'ApiKey' => $api,
            'Key' => "29500f07dc9d4df0b030e649f8a4c21b"
        );

        $json = json_encode($send);
        $ch = curl_init();
        curl_setopt($ch,CURLOPT_URL, "https://admin.smarthint.co/Account/NewOpenCartUser");
        curl_setopt($ch,CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, strlen($json));
        curl_setopt($ch,CURLOPT_POSTFIELDS, $json);
        $res = curl_exec($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        if ($httpcode<200 || $httpcode>300) {
            $this->log($send);
        }
        
        $mydata = json_decode($res);
        if( isset( $mydata->SHCode ) ){

            //Cria tabela para armazenar dados da Smarthint
            $this->db->query("  
                CREATE TABLE IF NOT EXISTS smarthint (
                DATE DATETIME,
                SHCODE VARCHAR(11),
                SHTOKEN VARCHAR(32));
            ");

            //Insere no banco Token SH 
            $query = "INSERT INTO smarthint (DATE, SHCODE, SHTOKEN) VALUES ('".date('Y-m-d H:i:s')."', '".$mydata->SHCode."', '".$mydata->Token."')";
            $this->db->query($query);
            return true;
        }
        return false;
    }

    public function processProducts(){
        $this->load->model('catalog/product');
        $data = array();
        $results = $this->model_catalog_product->getProducts($data);

        foreach ($results as $result) {
            $id = $result['product_id'];
            $product = $this->getProcuct($id);
            $this->do_curl_request("https://api.smarthint.co/api/Product/", $product);
        }
    }

    public function productHandler($event, $item) {
        $product = $this->getProcuct($item[0]);
        $this->do_curl_request("https://api.smarthint.co/api/Product/", $product);
    }

    public function getProcuct($id){
        
        //Recupera produto
        $this->load->model('catalog/category');
        $this->load->model('catalog/product');
        $this->load->model('catalog/manufacturer');
        $item = $this->model_catalog_product->getProduct($id);
        $catg = $this->model_catalog_product->getProductCategories($id);
        $imgs = $this->model_catalog_product->getProductImages($id);
        $salep = $this->model_catalog_product->getProductSpecials($id);
        $brand = $this->model_catalog_manufacturer->getManufacturer($item["manufacturer_id"]);

        //Obtem categorias
        $categs = array();
        foreach ($catg as $category)
            $categs[] = $this->model_catalog_category->getCategory($category["category_id"])["name"];

        //Obtem StockStatus
        if ($item["quantity"] != "0" && $item["status"] == "1" ){
            $stockStatus = "in stock";
        }
        else{
            $stockStatus = "out of stock";
        }

        //Obtem tamanho imagens
        $hei= $this->config->get('theme_default_image_product_height');
        $wid= $this->config->get('theme_default_image_product_width');
        $images = array();

        //Obtem imagem principal
        $this->load->model('tool/image');
        if ($item["image"]) {
            $image = $this->model_tool_image->resize($item["image"], $wid, $hei);
        } else {
            $image = '';
        }

        //Imagem secundária
        $images = array();
        $this->load->model('tool/image');
        if ($imgs) {
            $images[] = $this->model_tool_image->resize($imgs[0]["image"], $wid, $hei);
        }

        //Link
        $url = new Url(HTTP_CATALOG, $this->config->get('config_secure') ? HTTP_CATALOG : HTTPS_CATALOG);
        if ($this->config->get('config_seo_url')) {
            $productLink = $this->rewrite($url->link('product/product', 'product_id=' . $item["product_id"]));
        }else{
            $productLink = $url->link('product/product', 'product_id=' . $item["product_id"]);  
        }
        
        //SalePrice
        $saleprice = (isset($salep["price"])) ? $salep["price"] : null;
        $realBrand = (isset($brand["name"])) ? $brand["name"] : null;

        //Product config
        $product = array(
            'ProductId' => $item["product_id"],
            'Mpn' => $item["mpn"], 
            'Sku' => $item["sku"], 
            'Link' => $this->html_entity_decode($productLink),
            'Title' => $this->html_entity_decode($item["name"]),
            'Description' => $this->html_entity_decode($item["description"]),
            'Price' => $item["price"], 
            'SalePrice' => $saleprice, 
            'Condition' => "new", 
            'CreatedDate' => $item["date_available"],
            'BankSlipPrice' => "0",
            'Categories' => $categs,
            'ProductType' => end($categs),
            'Brand' => $realBrand,
            'ImageLink' => $this->html_entity_decode($image),
            'AdicionalImageLink' => $images,
            'Availability' => $stockStatus
        );
        return $product;
    }
    
    private function registerEvents(){
        $this->load->model('extension/event');
        $this->model_extension_event->deleteEvent("smarthint_module");

        //Listner para 2.0 ~ 2.2
        $this->model_extension_event->addEvent('smarthint_module', 'post.admin.product.add', 'extension/module/smarthint/productHandler');
        $this->model_extension_event->addEvent('smarthint_module', 'post.admin.product.edit', 'extension/module/smarthint/productHandler');
        $this->model_extension_event->addEvent('smarthint_module', 'post.admin.product.delete', 'extension/module/smarthint/productHandler');
        $this->model_extension_event->addEvent('smarthint_module', 'post.order.history.add', 'extension/module/smarthint/orderHandler');
        //Listner para 2.3 ~
        $this->model_extension_event->addEvent('smarthint_module', 'admin/model/catalog/product/addProduct/after', 'extension/module/smarthint/productHandler');
        $this->model_extension_event->addEvent('smarthint_module', 'admin/model/catalog/product/editProduct/after', 'extension/module/smarthint/productHandler');
        $this->model_extension_event->addEvent('smarthint_module', 'admin/model/catalog/product/copyProduct/after', 'extension/module/smarthint/productHandler');
        $this->model_extension_event->addEvent('smarthint_module', 'admin/model/catalog/product/deleteProduct/after', 'extension/module/smarthint/productHandler');
        $this->model_extension_event->addEvent('smarthint_module', 'catalog/model/checkout/order/addOrderHistory/after', 'extension/module/smarthint/orderHandler');
    }
    
    private function do_curl_request($url, $params=array()) {

        $json = json_encode($params);
        $ch = curl_init();
        $headers = $this->getHeader();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch,CURLOPT_URL, $url);
        curl_setopt($ch,CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, strlen($json));
        curl_setopt($ch,CURLOPT_POSTFIELDS, $json);
        $result = curl_exec($ch);

        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        if ($httpcode<200 || $httpcode>300) {
            $this->log($params);
        }
        return $result;
    }

    private function getHeader(){
        $auth = "";
        $response = $this->db->query("SELECT A.SHTOKEN FROM smarthint A ORDER BY A.DATE DESC LIMIT 1;");
        if ($response->num_rows > 0) {
            $auth = $response->row["SHTOKEN"];
        }

        return array(
            'Authorization: ' . $auth, 
            "Content-Type:application/json"
        );
    }

    public function rewrite($link) {
		$url_info = parse_url(str_replace('&amp;', '&', $link));

		$url = '';

		$data = array();

		parse_str($url_info['query'], $data);

		foreach ($data as $key => $value) {
			if (isset($data['route'])) {
				if (($data['route'] == 'product/product' && $key == 'product_id') || (($data['route'] == 'product/manufacturer/info' || $data['route'] == 'product/product') && $key == 'manufacturer_id') || ($data['route'] == 'information/information' && $key == 'information_id')) {
					$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "url_alias WHERE `query` = '" . $this->db->escape($key . '=' . (int)$value) . "'");

					if ($query->num_rows && $query->row['keyword']) {
						$url .= '/' . $query->row['keyword'];

						unset($data[$key]);
					}
				} elseif ($key == 'path') {
					$categories = explode('_', $value);

					foreach ($categories as $category) {
						$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "url_alias WHERE `query` = 'category_id=" . (int)$category . "'");

						if ($query->num_rows && $query->row['keyword']) {
							$url .= '/' . $query->row['keyword'];
						} else {
							$url = '';

							break;
						}
					}

					unset($data[$key]);
				}
			}
		}

		if ($url) {
			unset($data['route']);

			$query = '';

			if ($data) {
				foreach ($data as $key => $value) {
					$query .= '&' . rawurlencode((string)$key) . '=' . rawurlencode((is_array($value) ? http_build_query($value) : (string)$value));
				}

				if ($query) {
					$query = '?' . str_replace('&', '&amp;', trim($query, '&'));
				}
			}

			return $url_info['scheme'] . '://' . $url_info['host'] . (isset($url_info['port']) ? ':' . $url_info['port'] : '') . str_replace('/index.php', '', $url_info['path']) . $url . $query;
		} else {
			return $link;
		}
	}

    function html_entity_decode($string, $opt = 2) {
        $trans_tbl = get_html_translation_table(HTML_ENTITIES);
        $trans_tbl = array_flip ($trans_tbl);
        if ($opt & 1) { 
            $trans_tbl["&apos;"] = "'";
        }
        if (!($opt & 2)) {
            unset($trans_tbl["&quot;"]);
        }
        $response = strtr ($string, $trans_tbl);
        return strip_tags ($response);
    }
    
    private function log($data){
        $json = json_encode($data);
        $json = addslashes($json);
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type:application/json"));
        curl_setopt($ch,CURLOPT_URL, "https://webhook.logentries.com/noformat/logs/fae0c880-3aca-4de5-a85a-ec30356bc93f");
        curl_setopt($ch,CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, strlen($json));
        curl_setopt($ch,CURLOPT_POSTFIELDS, $json);
        $result = curl_exec($ch);
    }
}
?>