<?php
/**
 * Plugin Name: SmartHint - Smart Recommendations
 * Plugin URI: http://opencart.com/
 * Description: E-commerce Recommendation System increases the engagement and conversion rate by at least 35%. Like many online retailers, power up your sales serving to every customer as if they are unique. “For years, more than 35% of sales have been motivated by the recommendation system.” Jeff Bezos, CEO of Amazon.com.

 * Version: 1.0.0
 * Author: OpenCart
 * Author URI: https://opencart.com/
 * Developer: Carlos Renan Damarate
 * Developer URI: http://www.smarthint.co/
 *
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 */

class ControllerExtensionModuleSmartHint extends Controller {
    
    public function index(){
        $response = $this->db->query("select SHCODE from smarthint order by DATE DESC LIMIT 1;");
        if ($response->num_rows > 0) {
            $products = $this->cart->getProducts();
            $send = array(
                'code' => $response->row["SHCODE"],
                'cart' => count($products)
            );
            $this->response->setOutput(json_encode($send));
            return;
        }
        $this->response->setOutput(json_encode(""));
        return;
    }

    public function processProducts(){
        $response = $this->db->query("SELECT A.SHTOKEN FROM smarthint A ORDER BY A.DATE DESC LIMIT 1;");
        if ($response->num_rows > 0) {
            $this->load->model('catalog/product');
            $data = array();
            $results = $this->model_catalog_product->getProducts($data);

            foreach ($results as $result) {
                $id = $result['product_id'];
                $product = $this->getProcuct($id);
                $this->do_curl_request("https://api.smarthint.co/api/Product/", $product);
            }
        }else {
            $this->log(array(
                'Error' => 'Sem SHCode',
                'Mail' => $this->config->get('config_email'),
                'Domain' => (isset($url)) ? $url : "domain"
            ));
        }
    }

    public function orderHandler($route, $item) {

        $this->load->model('checkout/order'); // call this only if this model is not yet instantiated!
        $order = $this->model_checkout_order->getOrder($item[0]); // use the desired $orderId here
        
        //Product config
        $orderInfo = array(
            'OrderId' => $item[0],
            'Date' => $order['date_added'],
            'Freight' => "0",
            'Discount' => "0",
            'Total' => $order['total'],
            'Billing' => array (
                'Mode' => $order['payment_method'],
                'ModeId' => $order['payment_code'],
                'Installments' => '1',
                'InstallmentsValue' => $order['total']
            ),
            'Item' => []
        );

        $this->load->model('account/order');
        $products = $this->model_account_order->getOrderProducts($item[0]); 

        foreach ($products as $item) {
            $orderInfo['Item'][] = array(
                'Name' => $item['name'],
                'ProductId' => $item['product_id'],
                'Quantity' => $item['quantity'],
                'UnitPrice' => $item['price']
            );
        }
        
        $res = $this->do_curl_request("https://api.smarthint.co/api/Order/", $orderInfo); 
    }

    private function do_curl_request($url, $params=array()) {
        $json = json_encode($params);
        $ch = curl_init();
        $headers = $this->getHeader();
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch,CURLOPT_URL, $url);
        curl_setopt($ch,CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, strlen($json));
        curl_setopt($ch,CURLOPT_POSTFIELDS, $json);
        $result = curl_exec($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);
        if ($httpcode<200 || $httpcode>300) {
            $this->log(array(
                'status' => $httpcode,
                'parameters' => $params
            ));
        }
        return $result;
    }

    private function getHeader(){
        $auth = "";
        $response = $this->db->query("select SHTOKEN from smarthint order by DATE DESC LIMIT 1;");
        if ($response->num_rows > 0) {
            $auth = $response->row["SHTOKEN"];
        }

        return array(
            'Authorization: ' . $auth, 
            "Content-Type:application/json"
        );
    }

    
    public function getProcuct($id){
        
        //Recupera produto
        $this->load->model('catalog/category');
        $this->load->model('catalog/product');
        $this->load->model('catalog/manufacturer');
        $item = $this->model_catalog_product->getProduct($id);
        $catg = $this->model_catalog_product->getCategories($id);
        $imgs = $this->model_catalog_product->getProductImages($id);
        $brand = $this->model_catalog_manufacturer->getManufacturer($item["manufacturer_id"]);

        //Obtem categorias
        $categs = array();
        foreach ($catg as $category)
            $categs[] = $this->model_catalog_category->getCategory($category["category_id"])["name"];

        //Obtem StockStatus
        if ($item["quantity"] != "0" && $item["status"] == "1" ){
            $stockStatus = "in stock";
        }
        else{
            $stockStatus = "out of stock";
        }

        //Obtem tamanho imagens
        $hei= $this->config->get('theme_default_image_product_height');
        $wid= $this->config->get('theme_default_image_product_width');
        $images = array();

        //Obtem imagem principal
        $this->load->model('tool/image');
        if ($item["image"]) {
            $image = $this->model_tool_image->resize($item["image"], $wid, $hei);
        } else {
            $image = '';
        }

        //Imagem secundária
        $images = array();
        $this->load->model('tool/image');
        if ($imgs) {
            $images[] = $this->model_tool_image->resize($imgs[0]["image"], $wid, $hei);
        }

        //Link
        if (isset($this->request->server['HTTPS']) && (($this->request->server['HTTPS'] == 'on') || ($this->request->server['HTTPS'] == '1'))) {
			$base = $this->config->get('config_ssl');
		} else {
			$base = $this->config->get('config_url');
        }
        
        
        //SalePrice
        $realBrand = (isset($brand["name"])) ? $brand["name"] : null;

        //Link
        $url = new Url($base, $this->config->get('config_secure') ? $this->config->get('config_url') : $this->config->get('config_ssl'));
        if ($this->config->get('config_seo_url')) {
            $productLink = $this->rewrite($url->link('product/product', 'product_id=' . $item["product_id"]));
        }else{
            $productLink = $url->link('product/product', 'product_id=' . $item["product_id"]);
            
        }

        //Product config
        $product = array(
            'ProductId' => $item["product_id"],
            'Mpn' => $item["mpn"], 
            'Sku' => $item["sku"], 
            'Link' => $this->html_entity_decode($productLink),
            'Title' => $this->html_entity_decode($item["name"]),
            'Description' => $this->html_entity_decode($item["description"]),
            'Price' => $item["price"], 
            'SalePrice' => $item["special"],
            'Condition' => "new", 
            'CreatedDate' => $item["date_available"],
            'BankSlipPrice' => "0",
            'Categories' => $categs,
            'ProductType' => end($categs),
            'Brand' => $realBrand,
            'ImageLink' => $this->html_entity_decode($image),
            'AdicionalImageLink' => $images,
            'Availability' => $stockStatus
        );
        return $product;
    }

    public function rewrite($link) {
		$url_info = parse_url(str_replace('&amp;', '&', $link));

		$url = '';

		$data = array();

		parse_str($url_info['query'], $data);

		foreach ($data as $key => $value) {
			if (isset($data['route'])) {
				if (($data['route'] == 'product/product' && $key == 'product_id') || (($data['route'] == 'product/manufacturer/info' || $data['route'] == 'product/product') && $key == 'manufacturer_id') || ($data['route'] == 'information/information' && $key == 'information_id')) {
                    //echo "SELECT * FROM " . DB_PREFIX . "seo_url WHERE `query` = '" . $this->db->escape($key . '=' . (int)$value) . "'";
                    //url_alias
                    //echo $this->config->get('config_language_id') ;
                    $queryString = "SELECT * FROM " . DB_PREFIX . "seo_url WHERE `query` = '" . $this->db->escape($key . '=' . (int)$value) . "'";
                    $queryString .= " AND `language_id` = '". $this->config->get('config_language_id') ."';";
                    $query = $this->db->query($queryString);

					if ($query->num_rows && $query->row['keyword']) {
						$url .= '/' . $query->row['keyword'];

						unset($data[$key]);
					}
				} elseif ($key == 'path') {
					$categories = explode('_', $value);

					foreach ($categories as $category) {
						$query = $this->db->query("SELECT * FROM " . DB_PREFIX . "seo_url WHERE `query` = 'category_id=" . (int)$category . "'");

						if ($query->num_rows && $query->row['keyword']) {
							$url .= '/' . $query->row['keyword'];
						} else {
							$url = '';

							break;
						}
					}

					unset($data[$key]);
				}
			}
		}

		if ($url) {
			unset($data['route']);

			$query = '';

			if ($data) {
				foreach ($data as $key => $value) {
					$query .= '&' . rawurlencode((string)$key) . '=' . rawurlencode((is_array($value) ? http_build_query($value) : (string)$value));
				}

				if ($query) {
					$query = '?' . str_replace('&', '&amp;', trim($query, '&'));
				}
			}

			return $url_info['scheme'] . '://' . $url_info['host'] . (isset($url_info['port']) ? ':' . $url_info['port'] : '') . str_replace('/index.php', '', $url_info['path']) . $url . $query;
		} else {
			return $link;
		}
	}

    function html_entity_decode($string, $opt = 2) {
        
        $trans_tbl = get_html_translation_table(HTML_ENTITIES);
        $trans_tbl = array_flip ($trans_tbl);
        if ($opt & 1) { 
            $trans_tbl["&apos;"] = "'";
        }
        if (!($opt & 2)) {
            unset($trans_tbl["&quot;"]);
        }
        $response = strtr ($string, $trans_tbl);
        $buffer = str_replace(array("\r", "\n"), '', $response);
        return strip_tags ($buffer);
    }

    private function log($data){
        $json = json_encode($data);
        $json = addslashes($json);
        $ch = curl_init();
        $headers = $this->getHeader();
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type:application/json"));
        curl_setopt($ch,CURLOPT_URL, "https://webhook.logentries.com/noformat/logs/fae0c880-3aca-4de5-a85a-ec30356bc93f");
        curl_setopt($ch,CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch,CURLOPT_POST, strlen($json));
        curl_setopt($ch,CURLOPT_POSTFIELDS, $json);
        $result = curl_exec($ch);
    }
}
?>